<?php

namespace core\content;
use core\{application\Application, database\Database, utility\Utility, theme\Theme, field\Field, form\Form as F, user\User, route\Route, translation\Translation as T};

class Content {
	use Api, Form;

	function admin() {
		$total = Database::select('content')
			->fetch('count');
		
		$limit = 50;
		
		$pager = Route::get_pager($total, $limit, '/admin/content');

		$content = Database::select('content')
			->sort('created', 'DESC')
			->limit($limit, $pager['offset'])
			->fetch('assoc_all');

		foreach ($content as $key => $value) {
			$content_type = Database::select('content_types')
				->where('id', $value['type'])
				->fetch('assoc');

			$route_alias = Database::select('route_aliases')
				->data(['alias'])
				->where('entity_id', $value['id'])
				->where('type', 'content')
				->fetch('field');

			$content[$key]['route_alias'] = Route::render_route($route_alias);
			$content[$key]['type_name'] = $content_type['name'];
			$content[$key]['type_title'] = $content_type['title'];
		}

		$data = [
			'content' => $content,
			'pager' => $pager['html'],
			'material_add' => T::get('Добавить'),
			'material_types' => T::get('Типы'),
			'material_title' => T::get('Заголовок'),
			'type' => T::get('Тип'),
			'actions' => T::get('Действия'),
			'materials_no' => T::get('Материалы отсутствуют'),
			'change' => T::get('изменить'),
			'delete' => T::get('удалить'),
			
			'add_route' => Route::render_route('admin/content/add'),
			'types_route' => Route::render_route('admin/content/types'),
			'edit_route' => Route::render_route('admin/content/edit'),
			'delete_route' => Route::render_route('admin/content/delete'),
		];

		$result = (new Theme)->get_template(__DIR__, 'admin', $data);

		return $result;
	}

	//

	function add() {
		Utility::add_library('core/content/js/content.js');

		$result = '';
		$fields = [];
		if (!$_POST) {
			$result = $this->get_form_types_select();
		} else {
			if (isset($_POST['submit'], $_POST['title'])) {
				$data_prepare = $this->process_form();
				$data_prepare += [
					'type' => 'add'
				];

				$validate_result = $this->validate_form($data_prepare);

				if (!Utility::clear_array($validate_result)) {
					$result = $this->save_form($data_prepare);
					return $result;
				} else {
					$message = '';
					foreach ($validate_result as $key => $value) {
						$message .= $value ? implode('<br>', $value).'<br>' : '';
					}
					Utility::set_message($message, 'red');
				}
			}

			//

			$title = $_POST['title'] ?? '';
			$route_alias_auto = !isset($_POST['route_alias']) || isset($_POST['route_alias_auto']) ? 1 : 0;
			$route_alias = $_POST['route_alias'] ?? '';
			$body = $_POST['body'] ?? '';
			$content_type = !empty($_POST['content_type']) ? $_POST['content_type'] : 1;

			$fields = $this->get_form(compact('title', 'route_alias', 'route_alias_auto', 'body', 'content_type'), 'add', $content_type, 0, true);
			$result = (new F)->make($fields);
		}

		$result = [
			'raw' => compact('fields'),
			'body' => $result,
		];

		return $result;
	}

	function edit() {
		$id = Route::get_route_id();
		$object = $this->load($id);

		$result = '';
		$fields = [];
		if ($object) {
			$route_alias = Route::get_route_alias($id, 'content');

			if (isset($_POST['submit'])) {
				$data_prepare = $this->process_form();
				$data_prepare += [
					'type' => 'edit',
					'id' => $id,
					'uri_old' => $route_alias,
					'object_old' => $object,
				];

				$validate_result = $this->validate_form($data_prepare);

				if (!Utility::clear_array($validate_result)) {
					$result = $this->update_form($data_prepare);
					return $result;
				} else {
					$message = '';
					foreach ($validate_result as $key => $value) {
						$message .= $value ? implode('<br>', $value).'<br>' : '';
					}
					Utility::set_message($message, 'red');
				}
			}

			Utility::add_library('core/content/js/content.js');

			$title = $_POST['title'] ?? $object->title;
			$route_alias_auto = isset($_POST['route_alias_auto']) ? 1 : (isset($_POST['route_alias']) ? 0 : $object->route_alias_auto);
			$route_alias = $route_alias_auto ? $route_alias : ($_POST['route_alias'] ?? $route_alias);
			$body = $_POST['body'] ?? $object->body;
			$content_type = $object->type;

			$fields = $this->get_form(compact('title', 'route_alias', 'route_alias_auto', 'body', 'content_type'), 'edit', $object->type, $id, true);
			$result = (new F)->make($fields);
		}

		$result = [
			'raw' => compact('fields'),
			'body' => $result,
		];

		return $result;
	}

	function delete_item() {
		$id = Route::get_route_id();
		$data = $this->get_data($id);

		$result = '';
		if ($data) {
			if (!$_POST) {
				$result = Utility::get_form_delete();
			} else if (isset($_POST['cancel'])) {
				Route::go_to('admin/content');
			} else {
				Database::delete('content')
					->where('id',  $id)
					->execute();

				Database::delete('route_aliases')
					->where('entity_id',  $id)
					->where('type',  'content')
					->execute();

				(new Field)->get_fields_usage('delete', 'content', $data['type'], $id);

				Utility::delete_cache('content');
				Utility::set_message('success');
				Route::go_to('admin/content');
			}
		}

		return $result;
	}

	function show() {
		$user = new User;
		
		$id = Route::get_route_id();
		$data = $this->get_data($id);

		$result = '';
		if ($data) {
			Application::set_title($data['title']);
			$data['title'] = T::get($data['title']);
			
			$body = T::get($data['body']);	
			if (!T::check_language_default()) {
				$route_param = Route::get_route_language_param();
				$route_param_plus = Route::get_route_language_param('&');
				
				$dom_target = new \DOMDocument();
				@$dom_target->loadHTML($body);
				$xpath_target = new \DOMXPath($dom_target);
				$tags_target = $xpath_target->query('//a/@href');
				
				foreach ($tags_target as $key => $value) {
					if (mb_substr($value->nodeValue, 0, 1) == '/') {
						$value_temp = '"'.$value->nodeValue.'"';
						$route_param_temp = mb_strpos($value->nodeValue, '?') === false ? $route_param : $route_param_plus;	
						$value_source_temp = '"'.$value->nodeValue.$route_param_temp.'"';
						
						$body = str_replace($value_temp, $value_source_temp, $body);
					}
				}
			}			
			$data['body'] = $body;

			$data['type_name'] = $this->get_type_name($data['type']);
			$data['fields'] = (new Field)->get_fields_usage('show', 'content', $data['type'], $data['id']);

			$data['manage_links'] = $this->get_manage_links($data['id']);

			$result = (new Theme)->get_template(__DIR__, $data['type_name'], $data);
			if (!$result) {
				$fields = '';
				foreach ($data['fields'] as $key => $value) {
					if ($value) {
						$fields .= is_array($value) ? implode(', ', $value).', ' : $value.', ';
					}
				}
				$fields = rtrim($fields, ', ');

				$result = $data['manage_links'].$data['body'];
				$result .= $fields ? '<div class="margin-top-2">'.$fields.'</div>' : '';
			}
			
			if (!$user->get_access('administrator')) {
				Database::update('content')
					->data(['requests' => $data['requests'] + 1,])
					->where('id', $id)
					->execute();
			}
		} else if (Route::get_route(false, null, false) == '/') {
			$result = '<a href="'.Route::render_route('admin/content/add').'">'.T::get('Добавить материал').'</a>
				<div>'.T::get('Содержимое главной страницы еще не создано').'</div>';

		}

		$result = [
			'raw' => $data,
			'body' => $result,
		];

		return $result;
	}

	//

	function process_form() {
		$content_type = !empty($_POST['content_type']) ? $_POST['content_type'] : 1;
		$type_name = $this->get_type_name($content_type);

		$title = trim($_POST['title']);
		$body = trim($_POST['body']);

		if (!isset($_POST['route_alias_auto'])) {
			$route_alias = $_POST['route_alias'];
			$route_alias_auto = 0;
		} else {
			$title_translit = T::get_translit($title);
			$title_translit_parts = explode('-', $title_translit);

			$title_translit_parts_new = [];
			foreach ($title_translit_parts as $title_translit_part) {
				if ($title_translit_part !== '') {
					$title_translit_parts_new[] = $title_translit_part;
				}
			}

			$title_translit = implode('-', $title_translit_parts_new);

			$route_alias = str_replace('_', '-', $type_name).'/'.$title_translit;
			$route_alias_auto = 1;
		}

		$created = $changed = time();
		$values = compact('title', 'body', 'route_alias', 'route_alias_auto', 'type_name', 'created', 'changed');

		return $values;
	}

	function validate_form($data) {
		$route_alias = $data['route_alias'];
		$title = $data['title'];

		$reg_uri = '/^([a-z0-9-\/]{1,255})$/';
		$reg_title = '/^([а-яёa-z0-9-,№:?!=_()\. ]{1,255})$/iu';

		$route_aliases = Database::select('route_aliases')
			->where('alias', $route_alias)
			->fetch('count');

		$result = [];
		if (!preg_match($reg_title, $title)) {
			$result[] = T::get('Проверьте поле "Заголовок"');
		} else if ($route_aliases) {
			$url_nounique = T::get('Такой адрес уже существует');

			if ($data['type'] === 'add') {
				$result[] = $url_nounique;
			} else {
				if ($route_alias != $data['uri_old']) {
					$result[] = $url_nounique;
				}
			}
		} else if (!preg_match($reg_uri, $route_alias)) {
			$result[] = T::get('Сформированный адрес недопустим');
		}

		$content_type = !empty($_POST['content_type']) ? $_POST['content_type'] : 1;
		$fields_all = (new Field)->get_fields_usage('validate', 'content', $content_type);
		$result = array_merge(['body' => $result], $fields_all);

		return $result;
	}

	function save_form($data) {
		$content_type = !empty($_POST['content_type']) ? $_POST['content_type'] : 1;
		
		$id = Database::insert('content')
			->data([
				'type' => $content_type,
				'title' => $data['title'],
				'body' => $data['body'],
				'created' => $data['created'],
				'changed' => 0,
			])
			->fetch('field');

		Database::insert('route_aliases')
			->data([
				'auto' => $data['route_alias_auto'],
				'alias' => $data['route_alias'],
				'type' => 'content',
				'entity_id' => $id,
			])
			->execute();

		(new Field)->get_fields_usage('save', 'content', $content_type, $id);

		Utility::delete_cache('content');
		Utility::set_message('Материал успешно добавлен', 'green');

		$header_part = $data['route_alias'] == '/' ? '' : $data['route_alias'];
		Route::go_to($header_part);

		$result = [
			'raw' => [
				'content_id' => $id,
			],
		];

		return $result;
	}

	function update_form($data) {
		$content_type = !empty($_POST['content_type']) ? $_POST['content_type'] : 1;
		
		$object_old = $data['object_old'];
		T::update_strings($data['title'], $object_old->title);
		T::update_strings($data['body'], $object_old->body);
		
		Database::update('content')
			->data([
				'title' => $data['title'],
				'body' => $data['body'],
				'changed' => $data['changed'],
			])
			->where('id',  $data['id'])
			->execute();

		Database::update('route_aliases')
			->data([
				'auto' => $data['route_alias_auto'],
				'alias' => $data['route_alias'],
			])
			->where('entity_id', $data['id'])
			->where('type',  'content')
			->execute();

		(new Field)->get_fields_usage('update', 'content', $content_type, $data['id']);

		Utility::delete_cache('content');
		Utility::set_message('success');

		$header_part = $data['route_alias'] == '/' ? '' : $data['route_alias'];
		Route::go_to($header_part);

		$result = [
			'raw' => [
				'content_id' => $data['id'],
			],
		];

		return $result;
	}

	//

	function admin_types() {
		$content_types = Database::select('content_types')
			->fetch('assoc_all');

		$data = [
			'content_types' => $content_types,
			'add' => T::get('Добавить'),
			'content' => T::get('Материалы'),
			'name' => T::get('Название'),
			'title' => T::get('Заголовок'),
			'action' => T::get('Действия'),
			'config' => T::get('настроить'),
			'edit' => T::get('изменить'),
			'delete' => T::get('удалить'),			
			'empty' => T::get('Еще не добавлено ни одного типа материалов'),
			
			'content_route' => Route::render_route('admin/content'),
			'add_route' => Route::render_route('admin/content/types/add'),
			'config_route' => Route::render_route('admin/field/list/content'),
			'edit_route' => Route::render_route('admin/content/types/edit'),
			'delete_route' => Route::render_route('admin/content/types/delete'),
		];

		$result = (new Theme)->get_template(__DIR__, 'admin_types', $data);

		return $result;
	}

	function add_type() {
		if (isset($_POST['submit'])) {
			$data = [
				'name' => $_POST['name'],
				'title' => $_POST['title'],
			];

			$validate_result = $this->add_type_validate($data);

			if (!Utility::clear_array($validate_result)) {
				$this->add_type_save($data);
			} else {
				$message = '';
				foreach ($validate_result as $key => $value) {
					$message .= $value ? implode('<br>', $value).'<br>' : '';
				}
				Utility::set_message($message, 'red');
			}
		}

		$name = $_POST['name'] ?? '';
		$title = $_POST['title'] ?? '';

		$result = $this->get_form_types(compact('name', 'title'), 'add');

		return $result;
	}

	function edit_type() {
		$id = Route::get_route_id();
		$data = $this->get_data($id, 'content_types');

		$result = '';
		if ($data) {
			if (isset($_POST['submit'])) {
				$data = [
					'id' => $id,
					'name' => $data['name'],
					'title' => $_POST['title'],
				];

				$this->edit_type_save($data);
			}

			$name = $_POST['name'] ?? $data['name'];
			$title = $_POST['title'] ?? $data['title'];

			$result = $this->get_form_types(compact('name', 'title'), 'edit');
		}

		return $result;
	}

	function delete_type() {
		$id = Route::get_route_id();
		$data = $this->get_data($id, 'content_types');

		$result = '';
		if ($data) {
			if (!$_POST) {
				$result = Utility::get_form_delete('Также будут удалены все связанные материалы, поля и их данные');
			} else if (isset($_POST['cancel'])) {
				Route::go_to('admin/content/types');
			} else {
				$r_content = Database::select('content')
					->where('type', $data['id'])
					->fetch('assoc_all');

				foreach ($r_content as $row) {
					Database::delete('route_aliases')
						->where('entity_id', $row['id'])
						->where('type', 'content')
						->execute();
				}

				Database::delete('content')
					->where('type', $data['id'])
					->execute();

				Database::delete('content_types')
					->where('id', $id)
					->execute();

				(new Field)->get_fields_usage('drop', 'content', $data['id']);

				Database::delete('field_usage')
					->where('type', 'content')
					->where('subtype', $data['id'])
					->execute();

				Utility::set_message('success');
				Route::go_to('admin/content/types');
			}
		}

		return $result;
	}

	function add_type_validate($data) {
		$r_content_type = Database::select('content_types')
			->data(['name'])
			->where('name', $data['name'])
			->fetch('field');

		$result = [];
		if ($r_content_type) {
			$result[] = T::get('Такое название занято');
		}

		return $result;
	}

	function add_type_save($data) {
		Database::insert('content_types')
			->data([
				'name' => $data['name'],
				'title' => $data['title'],
			])
			->execute();

		Utility::set_message('success');
		Route::go_to('admin/content/types');
	}

	function edit_type_save($data) {
		Database::update('content_types')
			->data([
				'title' => $data['title'],
			])
			->where('id', $data['id'])
			->execute();

		Utility::set_message('success');
		Route::go_to('admin/content/types');
	}
}
