<?php

namespace core\field;
use core\{application\Application, utility\Utility, database\Database, route\Route, translation\Translation as T};

class Field {
	use Api, Form;

	function show_list() {
		$uri_parts = Route::get_route(true);
		$entity = $uri_parts[3];
		$entity_sub = $uri_parts[4];

		$table = $back = '';
		$entity_sub_data = [];
		if ($entity == 'content') {
			$table = 'content_types';
			$back = '/types';
		} else if ($entity == 'user') {
			$table = 'user_role';
			$back = '/roles';
		} else if ($entity == 'taxonomy') {
			$table = 'taxonomy_vocabulary';
		}

		if ($table) {
			$entity_sub_data = Database::select($table)
				->where('id', $entity_sub)
				->fetch('assoc');
		}

		$back = Route::render_route('admin/'.$entity.$back);

		$result = '';
		if ($entity_sub_data) {
			Application::set_title('Список полей "'.$entity.' ('.$entity_sub_data['name'].')"');

			$header = '<a href="'.$back.'">'.T::get('Вернуться назад').'</a>';

			$field = Database::select('field')
				->where('status', 1)
				->fetch('assoc_all', 'id');

			$r_field_usage = Database::select('field_usage')
				->where('field_id', array_keys($field))
				->where('type', $entity)
				->where('subtype', $entity_sub_data['id'])
				->sort('heft', 'ASC')
				->fetch('assoc_all');

			if ($r_field_usage) {
				$table_value = '';
				foreach ($r_field_usage as $row) {
					$required = T::get($row['required'] ? 'Да' : 'Нет');

					$table_value .= '<tr>
							<td>'.$field[$row['field_id']]['title'].'</td>
							<td>'.$row['name'].'</td>
							<td>'.$required.'</td>
							<td>'.$row['title'].'</td>
							<td>'.$row['description'].'</td>
							<td>'.$row['heft'].'</td>
							<td class="actions">
								<a class="margin-right" href="'.Route::render_route('admin/field/config/'.$entity.'/'.$entity_sub.'/'.$row['id']).'">'.T::get('настроить').'</a>
								<a href="'.Route::render_route('admin/field/drop/'.$entity.'/'.$entity_sub.'/'.$row['id']).'">'.T::get('удалить').'</a>
							</td>
						</tr>';
				}

				$list = '<table class="admin-table">
					<tr>
						<th class="width-20">'.T::get('Тип поля').'</th>
						<th>'.T::get('Название').'</th>
						<th>'.T::get('Обязательное').'</th>
						<th>'.T::get('Заголовок').'</th>
						<th class="width-30">'.T::get('Описание').'</th>
						<th>'.T::get('Вес').'</th>
						<th>'.T::get('Действия').'</th>
					</tr>'.$table_value.'</table>';
			} else {
				$list = '<div class="margin-top">'.T::get('Нет добавленных полей').'</div>';
			}

			$r_field = Database::select('field')
				->where('status', 1)
				->fetch('assoc_all');

			$fields = '';
			foreach ($r_field as $field) {
				$fields .= '<option value="'.$field['id'].'">'.$field['title'].'</option>';
			}

			$field_create = '<div class="margin-top">'.T::get('Нет доступных полей для добавления').'</div>';
			if ($fields) {
				$field_create = '<div class="margin-top">
						<form action="'.Route::render_route('admin/field/create/'.$entity.'/'.$entity_sub).'" method="post">
							<label class="pointer" for="select_field">'.T::get('Выберите поле для добавления').': </label>
							<select id="select_field" name="select_field">'.$fields.'</select>
							<input type="submit" class="margin-left" value="'.T::get('Добавить').'">
						</form>
					</div>';
			}

			$result = $header.$list.$field_create;
		}

		return $result;
	}

	//

	function create() {
		$select_field = $_POST['select_field'] ?? '';

		$result = $this->get_fields_usage('create', '', '', 0, $select_field, true);

		return $result;
	}

	function create_save($options = []) {
		$uri_parts = Route::get_route(true);
		$entity = $uri_parts[3];
		$entity_sub = $uri_parts[4];

		$result_field = Database::select('field')
			->where('id', $_POST['select_field'])
			->where('status', 1)
			->fetch('assoc');

		$required = isset($_POST['required']) ? 1 : 0;

		Database::insert('field_usage')
			->data([
				'field_id' => $result_field['id'],
				'type' => $entity,
				'subtype' => $entity_sub,
				'name' => $_POST['name'],
				'required' => $required,
				'title' => $_POST['title'],
				'description' => $_POST['description'],
				'quantity' => $_POST['quantity'] ?? 1,
				'heft' => $_POST['heft'],
				'options' => serialize($options),
			])
			->execute();

		Utility::set_message('success');
		Route::go_to('admin/field/list/'.$entity.'/'.$entity_sub);
	}

	//

	function config() {
		$uri_parts = Route::get_route(true);
		$entity = $uri_parts[3];
		$entity_sub = $uri_parts[4];
		$id = $uri_parts[5];

		$field_usage = Database::select('field_usage')
			->where('id', $id)
			->fetch('count');

		$result = '';
		if ($field_usage) {
			$result = $this->get_fields_usage('config', '', '', 0, $id, true);
		}

		return $result;
	}

	function config_save($options = []) {
		$uri_parts = Route::get_route(true);
		$entity = $uri_parts[3];
		$entity_sub = $uri_parts[4];
		$id = $uri_parts[5];

		$required = isset($_POST['required']) ? 1 : 0;

		Database::update('field_usage')
			->data([
				'required' => $required,
				'title' => $_POST['title'],
				'description' => $_POST['description'],
				'quantity' => $_POST['quantity'] ?? 1,
				'heft' => $_POST['heft'],
				'options' => serialize($options),
			])
			->where('id', $id)
			->execute();

		Utility::set_message('success');
		Route::go_to('admin/field/list/'.$entity.'/'.$entity_sub);
	}

	//

	function drop() {
		$uri_parts = Route::get_route(true);
		$entity = $uri_parts[3];
		$entity_sub = $uri_parts[4];
		$id = $uri_parts[5];

		$field_usage = Database::select('field_usage')
			->where('id', $id)
			->fetch('assoc');

		if (!$_POST) {
			$result = '';
			if ($field_usage) {
				$result = Utility::get_form_delete();
			}

			return $result;
		} else if (isset($_POST['cancel'])) {
			Route::go_to('admin/field/list/'.$entity.'/'.$entity_sub);
		} else {
			$this->get_fields_usage('drop', '', '', 0, $id, true);

			Database::delete('field_usage')
				->where('id', $id)
				->execute();

			Utility::set_message('success');
			Route::go_to('admin/field/list/'.$entity.'/'.$entity_sub);
		}
	}
}
