<?php

namespace core\translation;
use core\{utility\Utility, database\Database};

class Translation {
	use Api, Data;
	
	static function get($source) {
		$result = '';
		$source = trim($source);
		
		if ($source) {
			$language = self::get_language();
			
			if (Database::check()) {
				$strings = Database::select('strings_source')
					->data(['id', 'source',])
					->where('source', $source)
					->fetch('assoc_map_all');
				
				$id = 0;
				foreach ($strings as $key => $value) {
					if ($value == $source) {
						$id = $key;
						break;
					}
				}
				
				if ($id) {
					if ($language != 'ru') {
						$result = Database::select('strings_target')
							->data(['target',])
							->where('id', $id)
							->where('language', $language)
							->fetch('field');
					}
				} else {
					if ($language == 'ru') {
						Database::insert('strings_source')
							->data(['source' => $source,])
							->execute();
					}
				}
			} else {
				if ($language == 'ru') {
					$result = $source;
				} else {
					$GLOBALS['eqsash']['strings'] = $GLOBALS['eqsash']['strings'] ?? self::get_strings($language);
					$result = $GLOBALS['eqsash']['strings'][$language][$source] ?? '';
				}
			}
			
			$result = $result ?: $source;
		}

		return $result;
	}

	static function get_languages() {
		$languages = [
			'ru' => 'ru',
		];

		$files = glob(EQSASH_ROOT.'/others/strings/*.txt');
		foreach ($files as $key => $value) {
			$language = Utility::get_string_parts(basename($value, '.txt'), '-', -1);
			$languages[$language] = $language;
		}

		return $languages;
	}

	static function set_language($language) {
		$language = $_GET['l'] ?? $_POST['l'] ?? $language;
		
		if (in_array($language, self::get_languages())) {
			$GLOBALS['eqsash']['language'] = $language;
		}
	}

	static function get_language() {
		$language = $GLOBALS['eqsash']['language'] ?? '';

		return $language;
	}

	static function check_language_default() {
		$language_default = self::get_language() == self::get_language_default();

		return $language_default;
	}

	static function get_language_default() {
		$language_default = Data::$language_default;

		return $language_default;
	}

	static function check_language_change() {
		$result = false;
		
		$language = self::get_language();
		$language_store_file = __DIR__.'/'.Data::$language_store_file;
		
		if (isset($GLOBALS['language_store'])) {
			$language_store = $GLOBALS['language_store'];
		} else {
			$language_store = file_get_contents($language_store_file);
			$GLOBALS['language_store'] = $language_store;
		}
		
		if ($language != $language_store) {
			file_put_contents($language_store_file, $language);
			$result = true;
		}
		
		return $result;
	}
	
	static function import_strings($language = '', $file_source = '', $file_target = '') {		
		$strings = self::get_strings($language, $file_source, $file_target);
		$strings = $strings[$language] ?? [];
		
		foreach ($strings as $source => $target) {
			$strings_source = Database::select('strings_source')
				->data(['id', 'source',])
				->where('source', $source)
				->fetch('assoc_map_all');
			
			$id = 0;
			foreach ($strings_source as $key => $value) {
				if ($value == $source) {
					$id = $key;
					break;
				}
			}
			
			if (!$id) {
				$id = Database::insert('strings_source')
					->data(['source' => $source,])
					->fetch('field');
			}
			
			if ($id) {
				$result = Database::select('strings_target')
					->data(['target',])
					->where('id', $id)
					->where('language', $language)
					->fetch('field');
					
				if ($target != strip_tags($target)) {
					$dom_source = new \DOMDocument();
					@$dom_source->loadHTML("\xEF\xBB\xBF".$source);
					$xpath_source = new \DOMXPath($dom_source);
					$tags_source = $xpath_source->query('//*/@*');
					
					$dom_target = new \DOMDocument();
					@$dom_target->loadHTML($target);
					$xpath_target = new \DOMXPath($dom_target);
					$tags_target = $xpath_target->query('//*/@*');

					if (count($tags_source) == count($tags_target)) {
						foreach ($tags_target as $key => $value) {
							if (!in_array($value->name, Data::$attributes_stop)) {
								$value_temp = '"'.$value->nodeValue.'"';
								$value_source_temp = '"'.$tags_source[$key]->nodeValue.'"';
								
								$target = str_replace($value_temp, $value_source_temp, $target);
							}
						}
					} else {
						foreach ($tags_target as $key => $value) {
							if (!in_array($value->name, Data::$attributes_stop)) {
								$value_temp = '"'.$value->nodeValue.'"';
								$value_clean_temp = str_replace(' ', '', $value_temp);
								
								$target = str_replace($value_temp, $value_clean_temp, $target);
							}
						}
					}
					
					foreach (Data::$tags_replace as $key => $value) {
						foreach ($value as $key2 => $value2) {
							$target = str_replace($value2, $key, $target);
						}
					}
				}
				
				if ($result) {
					Database::update('strings_target')
						->data(['target' => $target,])
						->where('id', $id)
						->where('language', $language)
						->execute();
				} else {
					Database::insert('strings_target')
						->data(['id' => $id, 'language' => $language, 'target' => $target,])
						->execute();
				}
			}
		}
	}

	static function get_translit($string) {
		$string = strtr(mb_strtolower($string), Data::$converter);
		$string = preg_replace('/[^a-z0-9-]/u', '', trim($string, '-'));

		return $string;
	}
}
