<?php

namespace core\utility;
use core\{database\Database, theme\Theme, user\User, form\Form, field\Field, content\Content, module\Module, route\Route, translation\Translation as T};

class Utility extends Library {
	use Data;

	static function show_service($type = 'not', $header = true) {
		$map = [
			'not' => [
				'message' => T::get('Не найдено'),
				'header' => '404 Not Found',
			],
			'stop' => [
				'message' => T::get('Доступ запрещен'),
				'header' => '403 Forbidden',
			],
			'error' => [
				'message' => T::get('Произошла ошибка'),
			],
		];

		$errors = (new User)->get_access('administrator') ? Utility::get_errors() : [];

		$header_code = $map[$type]['header'] ?? '';
		if ($header && $header_code) {
			header('HTTP/1.1 '.$header_code);
			header('Status: '.$header_code);
		}

		$home_route = Route::render_route('');
		$message = $map[$type]['message'];
		$result = (new Theme)->get_template(__DIR__, 'service', compact('message', 'errors', 'home_route'));

		return $result;
	}

	static function set_message($message, $color = 'green') {
		if ($message == 'success') {
			$message = 'Операция успешно выполнена';
			$color = 'green';
		} else if ($message == 'error') {
			$message = 'Операция не выполнена';
			$color = 'red';
		}

		$_SESSION[$color.'_message'] = T::get($message);
	}

	static function get_messages($clear = false) {
		$messages = [];

		if (isset($_SESSION['green_message'])) {
			$messages['green'][] = $_SESSION['green_message'];
		}

		if (isset($_SESSION['red_message'])) {
			$messages['red'][] = $_SESSION['red_message'];
		}

		if ($clear) {
			unset($_SESSION['green_message'], $_SESSION['red_message']);
		}

		return $messages;
	}

	static function set_libraries($path, $weight = 0) {
		$files = array_merge(glob($path.'/*.js'), glob($path.'/*.css'));
		foreach ($files as $file) {
			$uri = Route::get_path($file);
			self::add_library($uri, $weight);
		}
	}

	static function add_library($source, $weight = 0, $type = '', $attributes = []) {
		$file_extension = Utility::get_file_extension($source);
		$type = $type ?: $file_extension;
		$local = mb_substr($source, 0, 4) != 'http';
		$embed = $local && !in_array($file_extension, ['css', 'js']);

		$attributes_html = '';
		foreach ($attributes as $key => $value) {
			$attributes_html .= ' '.$key.'="'.$value.'"';
		}

		$code = $source;
		if ($embed) {
			$code = $type == 'css' ? '<style>'.$source.'</style>' : '<script>'.$source.'</script>';
		} else {
			$link = $local ? Route::render_route($source) : $source;
			$code = $type == 'css' ? '<link rel="stylesheet" href="'.$link.'"'.$attributes_html.'>' : '<script src="'.$link.'"'.$attributes_html.'></script>';
		}

		$libraries = $GLOBALS['eqsash']['libraries'] ?? [];
		$libraries[$type][$source] = compact('weight', 'code', 'local');

		foreach ($libraries as $key => $value) {
			uasort($value, function ($one, $two) {
				return $one['weight'] > $two['weight'] ? 1 : ($one['weight'] < $two['weight'] ? -1 : 0);
			});
			$libraries[$key] = $value;
		}

		$GLOBALS['eqsash']['libraries'] = $libraries;
	}

	static function send_mail($to = '', $subject = '', $text = '', $mode = 'free', $files = [], $from = '') {
		$settings = self::get_settings();
		$system_name = $settings['main']['name'] ?? Route::get_domain();
		$from = $from ?: ($settings['main']['email_from'] ?? 'example@example.com');
		$to = $to ?: ($settings['main']['email_to'] ?? 'example@example.com');

		if ($mode == 'admin' || $mode == 'user') {
			$subject = mb_strtolower($subject);
			$text = T::get('Здравствуйте! Для Вас есть уведомление о новом событии').': <b>'.$subject.'</b>.<br><br>'.$text.'<br><br>-----<br>'.T::get('С уважением, система').' <i>'.$system_name.'</i>.';
			$subject = $system_name.': '.$subject;
		}

		self::build_mail($to, $subject, $text, $from, $files);
	}

	static function get_settings($name = '', $section = '') {
		$path = EQSASH_ROOT.'/settings.php';
		$path = file_exists($path) ? $path : EQSASH_ROOT.'/core/application/application.settings';

		require $path;

		if ($name) {
			if ($section) {
				$settings = $settings[$section][$name];
			} else {
				foreach ($settings as $key => $value) {
					if (isset($value[$name])) {
						$settings = $value[$name];
						break;
					}
				}
			}
		} else if ($section) {
			$settings = $settings[$section];
		}

		return $settings;
	}

	static function set_settings($settings = [], $permission = 0) {
		$path = EQSASH_ROOT.'/settings.php';

		if ($settings) {
			$permission_current = 0400;
			if (file_exists($path)) {
				chmod($path, 0700);
			}

			$content = '<?php'.PHP_EOL.PHP_EOL.'$settings = '.var_export($settings, true).';'.PHP_EOL;
			file_put_contents($path, $content, LOCK_EX);

			chmod($path, $permission_current);
		}

		if (file_exists($path) && $permission) {
			chmod($path, $permission);
		}
	}

	static function get_form_delete($warning = '', $data = '') {
		$form = new Form;

		$warning = $warning ? '. '.$warning : '';
		$warning = '<div>'.T::get('Это действие не может быть отменено').$warning.'</div>';

		$fields = [];
		if ($data) {
			$fields[] = [
				'input' => [
					'type' => 'hidden',
					'name' => 'data',
					'value' => $data,
				],
			];
		}

		$result = $form->make($fields, ['submit', 'cancel'], false);
		$result = $warning.$result;

		return $result;
	}

	static function detect_eol_type($string, $count = 1) {
		$result = mb_strpos($string, "\r\n") !== false ? "\r\n" : (mb_strpos($string, "\r") !== false ? "\r" : "\n");
		
		if ($count == 2) {
			$result .= $result;
		} else if ($count == 3) {
			$result .= $result.$result;
		}
		
		return $result;
	}

	//

	static function set_data($name, $value) {
		$value = serialize($value);

		$data = Database::select('data')
			->data(['id'])
			->where('name', $name)
			->fetch('field');

		if (!$data) {
			Database::insert('data')
				->data(compact('name', 'value'))
				->execute();
		} else {
			Database::update('data')
				->data(compact('value'))
				->where('name', $name)
				->execute();
		}
	}

	static function get_data($name) {
		$data = Database::select('data')
			->data(['value'])
			->where('name', $name)
			->fetch('field');

		$data = $data ? unserialize($data) : [];

		return $data;
	}

	static function delete_data($name) {
		Database::delete('data')
			->where('name', $name)
			->execute();
	}

	//

	static function set_output_format($value = 'theme') {
		$GLOBALS['eqsash']['output_format'] = $value;
	}

	static function get_output_format() {
		$output = $GLOBALS['eqsash']['output_format'] ?? 'theme';

		return $output;
	}

	static function set_errors($section, $error, $method = 'default') {
		$errors = self::get_variable('errors');

		if ($method == 'merge') {
			$errors[$section] = array_merge($errors[$section] ?? [], $error);
		} else if ($method == 'replace') {
			$errors[$section] = $error;
		} else {
			$errors[$section][] = $error;
		}

		self::set_variable('errors', $errors);
	}

	static function get_errors($section = '') {
		$errors = self::get_variable('errors') ?: [];
		$errors = $section ? ($errors[$section] ?? []) : $errors;

		return $errors;
	}

	static function set_cache($cache) {
		$GLOBALS['eqsash']['cache'] = $cache;
	}

	static function set_buffer($buffer) {
		$GLOBALS['eqsash']['buffer'] = $buffer;
	}

	static function get_buffer($sort = false) {
		$buffer = $GLOBALS['eqsash']['buffer'] ?? [];

		if ($sort) {
			$buffer_sorted = [];
			foreach ($buffer as $key => $value) {
				foreach ($value as $key_two => $value_two) {
					$value_two['entity'] = $key;
					$buffer_sorted[] = $value_two;
				}
			}

			$buffer = Utility::sort_array($buffer_sorted, 'weight');
		}

		return $buffer;
	}

	static function add_buffer($entity, $data) {
		$buffer = self::get_buffer();
		$buffer[$entity][] = $data;
		self::set_buffer($buffer);
	}

	static function parse_buffer($entity, $full = false) {
		$buffer = self::get_buffer();

		$buffer_entity = $buffer[$entity] ?? [];
		$result = $full ? $buffer_entity : ($buffer_entity[0]['data']['raw'] ?? []);

		return $result;
	}

	static function sort_array($array, $field, $keys = false, $direction = 'default') {
		$function = $keys ? 'uasort' : 'usort';

		$function($array, function ($one, $two) use ($field, $direction) {
			return $one[$field] > $two[$field] ? ($direction == 'default' ? 1 : -1) : ($one[$field] < $two[$field] ? ($direction == 'default' ? -1 : 1) : 0);
		});

		return $array;
	}

	static function get_system_name() {
		$settings = self::get_settings();
		$system_name = $settings['main']['name'] ?? Route::get_domain();
		$system_name = mb_strtoupper(mb_strcut($host, 0, mb_strpos($host, '.')));

		return $system_name;
	}

	static function get_abbreviation($string) {
		$string = mb_strtoupper(preg_replace('/[^а-яa-z ]/iu', '', $string));
		$string_parts = explode(' ', $string);
		$string_part = $string_parts[0] ?? '';
		$string_part2 = $string_parts[1] ?? '';
		$abbreviation = mb_substr($string_part, 0, 1).mb_substr($string_part2, 0, 1);

		return $abbreviation;
	}

	static function unserialize_object_property($object, $property) {
		$data = isset($object->{$property}) ? unserialize($object->{$property}) : [];
		unset($object->{$property});
		foreach ($data as $key => $value) {
			$object->{$key} = $value;
		}

		return $object;
	}

	static function set_variable($variable, $value) {
		$GLOBALS['eqsash']['variables'][$variable] = $value;
	}

	static function get_variable($variable) {
		$variable = $GLOBALS['eqsash']['variables'][$variable] ?? [];

		return $variable;
	}

	static function unset_variable($variable) {
		unset($GLOBALS['eqsash']['variables'][$variable]);
	}

	static function clear_variables($variable) {
		unset($GLOBALS['eqsash']['variables']);
	}

	static function set_request_group($name, $options) {
		$group = [];
		foreach ($options as $key => $value) {
			$group[$name.'_'.$key] = $value;
		}

		return $group;
	}

	static function make_directory($directory, $permissions = 0777) {
		if (!is_dir($directory)) {
			mkdir($directory, $permissions, true);
		}
	}

	static function get_shutdown() {
		self::process_errors('', '', '', '');
	}

	static function process_errors($type, $message, $file, $line) {
		$settings = self::get_settings();
		
		if ($settings['mode'] == 'test') {
			if (!$message) {
				$error = error_get_last();
				if ($error) {
					extract($error);
				} else {
					return false;
				}
			}
			
			$type = Data::$error_types[$type] ?? $type;
			$data = compact('type', 'message', 'file', 'line');

			$directory = EQSASH_ROOT.'/'.Route::$path_data.'/logs';
			$file = $directory.'/errors.log';

			if (file_exists($file)) {
				$file_size = filesize($file);
				$flag = $file_size > 1000000 ? 0 : FILE_APPEND;
			} else {
				$flag = 0;
				self::make_directory($directory);
			}
			
			file_put_contents($file, print_r($data, true).PHP_EOL, $flag);
		}

		return false;
	}

	static function load_object($id, $entity, $fields = [], $fields_type = 'type') {
		$objects = self::load_object_multi($id, $entity, $fields, $fields_type);
		$object = $objects[$id] ?? [];

		return $object;
	}

	static function load_object_multi($ids, $entity, $fields = [], $entity_type = 'type', $all = false, $entity_type_data = []) {
		$objects_data = Database::select($entity);

		if (!$all) {
			$objects_data->where('id', $ids);
		}

		$objects_data = $objects_data->fetch('assoc_all', 'id');

		$route_aliases = Database::select('route_aliases')
			->data(['entity_id', 'auto', 'alias',])
			->where('type', $entity);

		if (!$all) {
			$route_aliases->where('entity_id', $ids);
		}

		$route_aliases = $route_aliases->fetch('object_all', 'entity_id');

		$objects = [];
		foreach ($objects_data as $key => $value) {
			$value['route_alias'] = $route_aliases[$key]->alias ?? '';
			$value['route_alias_auto'] = $route_aliases[$key]->auto ?? '';
			$entity_type_temp = $value[$entity_type] ?? array_keys($entity_type_data[$key] ?? []);
			$value[$entity_type] = $value[$entity_type] ?? $entity_type_data[$key] ?? [];

			if ($fields !== false && isset($value['id'])) {
				$fields_data = (new Field)->get_fields_usage('get', $entity, $entity_type_temp, $value['id'], 0, false, [], $fields);

				$fields_all = [];
				foreach ($fields_data as $key_two => $value_two) {
					$fields_all[$key_two] = $value_two;
				}
				$value = $value + $fields_all;
			}

			$objects[$key] = (object)$value;
		}

		return $objects;
	}

	//

	static function check_cache() {
		$cache = $GLOBALS['eqsash']['cache'] ?? true;

		return $cache;
	}

	static function get_cache($section, $name) {
		$result = false;
		if (self::check_cache()) {
			$cache_file = EQSASH_ROOT.'/'.Route::$path_cache.'/'.$section.'/'.$name.'.php';

			if (file_exists($cache_file)) {
				require $cache_file;
				$result = ${$name};
			}
		}

		return $result;
	}

	static function add_cache($section, $name, $data) {
		if (self::check_cache()) {
			$directory = EQSASH_ROOT.'/'.Route::$path_cache.'/'.$section;
			self::make_directory($directory);
			$cache_file = $directory.'/'.$name.'.php';

			$content = '<?php'.PHP_EOL.PHP_EOL.'$'.$name.' = '.var_export($data, true).';'.PHP_EOL;
			file_put_contents($cache_file, $content);
		}

		return $data;
	}

	static function delete_cache($section, $name = '') {
		if (self::check_cache()) {
			$directory = EQSASH_ROOT.'/'.Route::$path_cache.'/'.$section;

			if ($name) {
				self::delete_file($directory.'/'.$name.'.php');
			} else {
				self::delete_directory($directory);
			}
		}

		return true;
	}

	static function clear_cache() {
		Route::update_interim_files();
		self::delete_directory(EQSASH_ROOT.'/others/data/cache');
		self::delete_directory(Route::$path_files_temp);

		return true;
	}

	static function download_file($file, $die = false, $delete = false, $name = '') {
		if (file_exists($file)) {
			if (ob_get_level()) { 
				ob_end_clean(); 
			}
			
			header('Content-Description: File Transfer');
			header('Content-Disposition: attachment; filename='.($name ?: basename($file)));
			header('Expires: 0');
			header('Cache-Control: must-revalidate');
			header('Pragma: public');
			header('Content-Length: '.filesize($file));
			readfile($file);
		
			if ($delete) {
				unlink($file);
			}
		} else {
			Route::go('');
		}
		
		if ($die) {
			die;
		}
	}
}
